/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "adc.h"
#include "i2c.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include "dht11.h"
#include "sht30.h"
#include "ds18b20.h"
#include "core_json.h"
#include "oled.h"
#include "esp8266.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
#define FLAG_WIFI_CONNECTED     (1<<0)
#define FLAG_SOCK_CONNECTED     (1<<1)
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static int report_tempRH_json(void);
static int parser_led_json(char *json_string, int bytes);
static void proc_uart1_recv(void);
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  uint32_t            last_time = 0;
  unsigned char       buf[256];
  char                ipaddr[16];
  char                gateway[16];
  unsigned char       wifi_flag = 0;
  int                 rv;
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM6_Init();
  MX_TIM2_Init();
  MX_USART1_UART_Init();
  MX_ADC1_Init();
  MX_I2C2_Init();
  MX_USART2_UART_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  sysled_hearbeat();
  beep_start(2, 300);
  printf("Start BearKE1 5G NB-IoT Board Example Program v3.0\r\n");

  OLED_Init();
  OLED_ShowBanner(TIME_1S*2);

  esp8266_module_init();

  while (1)
  {
	  if( ! (wifi_flag&FLAG_WIFI_CONNECTED) )
	  {
		  if( esp8266_join_network("Router_Home2G", "n3tacc355") )
		  {
			  esp8266_module_init();
			  HAL_Delay(2000);
			  continue;
		  }

		  if( esp8266_get_ipaddr(ipaddr, gateway, sizeof(ipaddr) ) )
		  {
			  HAL_Delay(1000);
			  continue;
		  }

		  if( esp8266_ping_test(gateway) )
		  {
			  HAL_Delay(1000);
			  continue;
		  }

		  wifi_flag |= FLAG_WIFI_CONNECTED; /* set wifi connected flag */
	  }

	  if( ! (wifi_flag&FLAG_SOCK_CONNECTED) )
	  {
		  if( esp8266_sock_connect("192.168.2.103", 12345) )
		  {
			  HAL_Delay(1000);
			  continue;
		  }

		  wifi_flag |= FLAG_SOCK_CONNECTED; /* set socket connected flag */
	  }

	  if( (rv=esp8266_sock_recv(buf, sizeof(buf))) > 0 )
	  {
		  printf("ESP8266 socket receive %d bytes data: %s\n", rv, buf);
		  parser_led_json((char *)buf, rv);
	  }

	  if( time_after(HAL_GetTick(), last_time+3000) )
	  {
		  rv = report_tempRH_json();
		  if( rv > 0 )
		  {
			  printf("ESP8266 socket send message ok\n");
		  }
		  else if( rv < 0 )
		  {
			  printf("ESP8266 socket send message failure, rv=%d\n", rv);

			  wifi_flag &= ~FLAG_SOCK_CONNECTED; /* clear socket connected flag */

			  if( esp8266_ping_test(gateway) )
			  {
				  wifi_flag &= ~FLAG_WIFI_CONNECTED; /* clear wifi connected flag */
			  }
		  }

		  last_time = HAL_GetTick(); /*update last report time */
	  }

#if 0
	  proc_uart1_recv();

	  /* check report time arrive or not, use time_after() macro to avoid time overflow */
	  if( time_after(HAL_GetTick(), last_time+3000) )
	  {
		  /* start sample and report data to PC by serial port */
		  report_tempRH_json();

		  /*update last report time */
		  last_time = HAL_GetTick();
	  }
#endif
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  if (HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 1;
  RCC_OscInitStruct.PLL.PLLN = 20;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV7;
  RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV2;
  RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
int report_tempRH_json(void)
{
	char          buf[128];
    float         temperature, humidity=0.0;
    uint32_t      temp, humd;

    if ( SHT30_SampleData(&temperature, &humidity) < 0 )
    {
    	return 0;
    }

    memset(buf, 0, sizeof(buf));
    snprintf(buf, sizeof(buf), "{\"Temperature\":\"%.2f\", \"Humidity\":\"%.2f\"}", temperature, humidity);

    temp = (int)(temperature*100);
    humd = (int)(humidity*100);
    OLED_ShowTempHumdity(temp, humd, TIME_1S*2);

    //HAL_UART_Transmit(&huart1 , (uint8_t *)buf, strlen(buf), 0xFFFF);
    return esp8266_sock_send((uint8_t *)buf, strlen(buf));
}

int parser_led_json(char *json_string, int bytes)
{
   JSONStatus_t             result;
    char                     save;
    char                    *value;
    size_t                   valen;
    int                      i;

    printf("DBUG: Start parser JSON string: %s\r\n", json_string);

	result = JSON_Validate(json_string, bytes);

	/* JSON document is valid so far but incomplete */
	if( JSONPartial == result )
	{
		printf("WARN: JSON document is valid so far but incomplete!\r\n");
		return 0;
	}

	/* JSON document is not valid JSON */
	if( JSONSuccess != result )
	{
		printf("ERROR: JSON document is not valid JSON!\r\n");
		return -1;
	}

	/* Parser and set LED status */
	for(i=0; i<LedMax; i++)
	{
		result = JSON_Search( json_string, bytes,  leds[i].name, strlen(leds[i].name), &value, &valen);
		if( JSONSuccess == result )
		{
			 save = value[valen];
			 value[valen] = '\0';

			 if( !strncasecmp(value, "on", 2) )
			 {
				 printf("DBUG: turn %s on\r\n", leds[i].name);
				 turn_led(i, ON);
			 }
			 else if( !strncasecmp(value, "off", 3) )
			 {
				 printf("DBUG: turn %s off\r\n", leds[i].name);
				 turn_led(i, OFF);
			 }

			 value[valen] = save;
		}
	}

	return 1;
}

void proc_uart1_recv(void)
{
	  if( g_uart1_bytes > 0 )
	  {
		     HAL_Delay(200);
		     if( 0 != parser_led_json(g_uart1_rxbuf, g_uart1_bytes) )
		     {
		    	 clear_uart1_rxbuf();
		     }
	  }
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
